// Copyright (c) 2023 Tudor Stanciu

import { getUrlTemplates } from "../config";
import { TuitioClient } from "../client";
import type { TuitioLoginResponse } from "../types";

test("Get url templates", () => {
  const result = getUrlTemplates("https://test.com/api");
  expect(result).toHaveProperty("login");
  expect(result).toHaveProperty("logout");
  expect(result).toHaveProperty("userInfo");
  expect(result.login).toContain("https://test.com/api");
  expect(result.login).toContain("{username}");
  expect(result.login).toContain("{password}");
  expect(result.logout).toContain("{token}");
});

test("Tuitio client initialization", () => {
  const tuitioUrl = "https://test.com/api";
  const client = new TuitioClient(tuitioUrl);
  expect(client.tuitioUrl).toBe(tuitioUrl);
  expect(client.urlTemplates.login.startsWith(tuitioUrl)).toBe(true);
  expect(client.urlTemplates.logout.startsWith(tuitioUrl)).toBe(true);
});

test("Tuitio client initialization with null URL", () => {
  try {
    new TuitioClient(null);
  } catch (error: any) {
    expect(error.message).toContain("is not a valid URL");
  }
});

test("Tuitio client initialization with invalid URL", () => {
  try {
    new TuitioClient("test");
  } catch (error: any) {
    expect(error.message).toContain("is not a valid URL");
  }
});

test("Tuitio client fake authentication", async () => {
  const client = new TuitioClient("https://test.com/api");
  const spy = jest.spyOn(client, "login").mockImplementation(async (userName, password) => {
    const token = `mock-${userName}-${password}`;
    const authResult = <TuitioLoginResponse>{ result: { token, expiresIn: 600000 }, error: null };
    return authResult;
  });

  const loginResult = await client.login("user", "pass");

  expect(loginResult.result?.token).toBe("mock-user-pass");
  expect(loginResult.result?.expiresIn).toBe(600000);
  expect(loginResult.error).toBeNull;

  spy.mockRestore();
});
